/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  */
/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "main.h"
#include "eeprom.h"
#include "usb_device.h"
#include "usbd_audio.h"
#include "audio_in_task.h"
#include "ui_task.h"
#include "DWT_delay.h"
#include "usbd_customhid.h"
#include "usbd_composite.h"
#include "appjmp.h"

I2C_HandleTypeDef hi2c2;
DMA_HandleTypeDef hdma_i2c2_rx;
DMA_HandleTypeDef hdma_i2c2_tx;

SAI_HandleTypeDef hsai_in;
DMA_HandleTypeDef hdma_sai_in;

QueueHandle_t audio_in_queue = NULL;
QueueHandle_t ui_queue = NULL;

// EEPROM emulation
uint16_t VirtAddVarTab[NB_OF_VAR] = {0x5555, 0x6666, 0x7777};

void SystemClock_Config(void);
static void GPIO_Init(void);
static void DMA_Init(void);
static void I2C2_Init(void);
static void SAI2_Init(void);
static void GPIO_Init(void);
static void setup_EE( void );
static void setup_hw( void );
static void CPU_CACHE_Enable(void);
/* Prototypes for the standard FreeRTOS callback/hook functions implemented within this file. */
void vApplicationMallocFailedHook( void );
void vApplicationIdleHook( void );
void vApplicationStackOverflowHook( TaskHandle_t pxTask, char *pcTaskName );
void vApplicationTickHook( void );

/**
  * @brief  Write function for SWV output.
  * @retval int
  */
int _write(int file, char *ptr, int len)
{
	/* Implement your write code here, this is used by puts and printf for example */
	int i=0;
	for(i=0 ; i<len ; i++)
		ITM_SendChar((*ptr++));
	return len;
}

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
	if (dfu_mode == JUMP_TO_DFU) {
		dfu_mode = 0L;
		jumpTo(DFU_BOOT_ADDR);
	}
	setup_hw();
	vStartUITask();
	if (_cfg.audio_in) {
		vStartAudioInTask();
	}
	USB_DEVICE_Init();
	/* Start the tasks and timer running. */
	vTaskStartScheduler();

	for( ;; );
	return 0;
}

/**
  * @brief  Configure the MPU attributes .
  * @param  None
  * @retval None
  */
static void MPU_Config(void)
{
	MPU_Region_InitTypeDef MPU_InitStruct;

	/* Disable the MPU */
	HAL_MPU_Disable();
	/* Configure the MPU attributes for SRAM1 and SRAM2 */
	MPU_InitStruct.Enable = MPU_REGION_ENABLE;
	MPU_InitStruct.BaseAddress = 0x20010000;
	MPU_InitStruct.Size = MPU_REGION_SIZE_128KB;
	MPU_InitStruct.AccessPermission = MPU_REGION_FULL_ACCESS;
	MPU_InitStruct.IsBufferable = MPU_ACCESS_BUFFERABLE;
	MPU_InitStruct.IsCacheable = MPU_ACCESS_CACHEABLE;
	MPU_InitStruct.IsShareable = MPU_ACCESS_SHAREABLE;
	MPU_InitStruct.Number = MPU_REGION_NUMBER0;
	MPU_InitStruct.TypeExtField = MPU_TEX_LEVEL0;
	MPU_InitStruct.SubRegionDisable = 0x00;
	MPU_InitStruct.DisableExec = MPU_INSTRUCTION_ACCESS_ENABLE;
	HAL_MPU_ConfigRegion(&MPU_InitStruct);

	/* MPU attributes for dma buffers */
	MPU_InitStruct.Enable = MPU_REGION_ENABLE;
	MPU_InitStruct.BaseAddress = 0x20030000;
	MPU_InitStruct.Size = MPU_REGION_SIZE_64KB;
	MPU_InitStruct.AccessPermission = MPU_REGION_FULL_ACCESS;
	MPU_InitStruct.IsBufferable = MPU_ACCESS_NOT_BUFFERABLE;
	MPU_InitStruct.IsCacheable = MPU_ACCESS_NOT_CACHEABLE;
	MPU_InitStruct.IsShareable = MPU_ACCESS_SHAREABLE;
	MPU_InitStruct.Number = MPU_REGION_NUMBER1;
	MPU_InitStruct.TypeExtField = MPU_TEX_LEVEL0;
	MPU_InitStruct.SubRegionDisable = 0x00;
	MPU_InitStruct.DisableExec = MPU_INSTRUCTION_ACCESS_ENABLE;
	HAL_MPU_ConfigRegion(&MPU_InitStruct);

	/* Enable the MPU */
	HAL_MPU_Enable(MPU_PRIVILEGED_DEFAULT);
}

/**
  * @brief  CPU L1-Cache enable.
  * @param  None
  * @retval None
  */
static void CPU_CACHE_Enable(void)
{
	/* Enable I-Cache */
	SCB_EnableICache();

	/* Enable D-Cache */
	SCB_EnableDCache();
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
    RCC_OscInitTypeDef RCC_OscInitStruct = {0};
    RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
    RCC_PeriphCLKInitTypeDef PeriphClkInitStruct = {0};

    /* Configure LSE Drive Capability */
    HAL_PWR_EnableBkUpAccess();
    /* Configure the main internal regulator output voltage */
    __HAL_RCC_PWR_CLK_ENABLE();
    __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);
    /* Initializes the RCC Oscillators according to the specified parameters
     * in the RCC_OscInitTypeDef structure.
     */
    RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_HSE;
    RCC_OscInitStruct.HSEState = RCC_HSE_ON;
    RCC_OscInitStruct.LSIState = RCC_LSI_ON;
    RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
    RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
    RCC_OscInitStruct.PLL.PLLM = 25;
    RCC_OscInitStruct.PLL.PLLN = 432;
    RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
    RCC_OscInitStruct.PLL.PLLQ = 9;
    if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK) {
    	Error_Handler();
    }
    /* Activate the Over-Drive mode */
    if (HAL_PWREx_EnableOverDrive() != HAL_OK) {
    	Error_Handler();
    }
    /* Initializes the CPU, AHB and APB buses clocks */
    RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
    RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
    RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
    RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
    RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

    if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_7) != HAL_OK) {
    	Error_Handler();
    }
    PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_SAI2|RCC_PERIPHCLK_I2C2|RCC_PERIPHCLK_CLK48;
    PeriphClkInitStruct.PLLI2S.PLLI2SN = 295;
    PeriphClkInitStruct.PLLI2S.PLLI2SR = 2;
    PeriphClkInitStruct.PLLI2S.PLLI2SQ = 3;
    PeriphClkInitStruct.PLLI2SDivQ = 1;
    PeriphClkInitStruct.Sai2ClockSelection = RCC_SAI2CLKSOURCE_PIN;		// Use I2S_CKIN for SAI2
    PeriphClkInitStruct.I2c2ClockSelection = RCC_I2C2CLKSOURCE_PCLK1;
    PeriphClkInitStruct.Clk48ClockSelection = RCC_CLK48SOURCE_PLL;
    if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK) {
    	Error_Handler();
    }
}

/**
  * @brief I2C2 Initialization Function
  * @param None
  * @retval None
  */
static void I2C2_Init(void)
{
	/* I2C for I2S output */
	hi2c2.Instance = I2C2;
	hi2c2.Init.Timing = 0x10A60D20;	// Fast 400k/300/300
//	hi2c2.Init.Timing = 0x10E66A8D;	// Std 100k/300/300
//	hi2c2.Init.Timing = 0x20404768; // Std 100k/0/0
	hi2c2.Init.OwnAddress1 = 0;
	hi2c2.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
	hi2c2.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
	hi2c2.Init.OwnAddress2 = 0;
	hi2c2.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
	hi2c2.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
	hi2c2.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
	if (HAL_I2C_Init(&hi2c2) != HAL_OK) {
		Error_Handler();
	}
	/* Configure Analogue filter */
	if (HAL_I2CEx_ConfigAnalogFilter(&hi2c2, I2C_ANALOGFILTER_ENABLE) != HAL_OK) {
		Error_Handler();
	}
	/* Configure Digital filter */
	if (HAL_I2CEx_ConfigDigitalFilter(&hi2c2, 0) != HAL_OK) {
		Error_Handler();
	}
}

/**
  * @brief SAI2 Initialization Function
  * @param None
  * @retval None
  */
static void SAI2_Init(void)
{
	/* Slave RX */
	hsai_in.Instance = SAI2_Block_B;
	hsai_in.Init.AudioMode = SAI_MODESLAVE_RX;
	hsai_in.Init.Synchro = SAI_ASYNCHRONOUS;
	hsai_in.Init.OutputDrive = SAI_OUTPUTDRIVE_DISABLE;
	hsai_in.Init.FIFOThreshold = SAI_FIFOTHRESHOLD_EMPTY;
	hsai_in.Init.SynchroExt = SAI_SYNCEXT_OUTBLOCKB_ENABLE;
	hsai_in.Init.MonoStereoMode = SAI_STEREOMODE;
	hsai_in.Init.CompandingMode = SAI_NOCOMPANDING;
	hsai_in.Init.TriState = SAI_OUTPUT_NOTRELEASED;
	if (HAL_SAI_InitProtocol(&hsai_in, SAI_I2S_STANDARD, SAI_PROTOCOL_DATASIZE_32BIT, 2) != HAL_OK) {
		Error_Handler();
	}
}

/**
  * Enable DMA controller clock
  */
static void DMA_Init(void)
{
	/* DMA controller clock enable */
	__HAL_RCC_DMA2_CLK_ENABLE();
	__HAL_RCC_DMA1_CLK_ENABLE();

	/* DMA interrupt init */
	/* DMA1_Stream2_IRQn interrupt configuration = I2C2*/
	HAL_NVIC_SetPriority(DMA1_Stream2_IRQn, 8, 0);
	HAL_NVIC_EnableIRQ(DMA1_Stream2_IRQn);
	/* DMA1_Stream7_IRQn interrupt configuration = I2C2*/
	HAL_NVIC_SetPriority(DMA1_Stream7_IRQn, 8, 0);
	HAL_NVIC_EnableIRQ(DMA1_Stream7_IRQn);
	/* DMA2_Stream6_IRQn interrupt configuration = SAI2B*/
	HAL_NVIC_SetPriority(DMA2_Stream6_IRQn, 5, 0);
	HAL_NVIC_EnableIRQ(DMA2_Stream6_IRQn);
}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void GPIO_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	/* GPIO Ports Clock Enable */
	__HAL_RCC_GPIOA_CLK_ENABLE();
	__HAL_RCC_GPIOB_CLK_ENABLE();
	__HAL_RCC_GPIOC_CLK_ENABLE();
	__HAL_RCC_GPIOD_CLK_ENABLE();
	__HAL_RCC_GPIOE_CLK_ENABLE();
	__HAL_RCC_GPIOF_CLK_ENABLE();
	__HAL_RCC_GPIOG_CLK_ENABLE();

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(I2SIN_OUT1_PORT, I2SIN_OUT1_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(I2SIN_OUT2_PORT, I2SIN_OUT2_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(I2SIN_OUT3_PORT, I2SIN_OUT3_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(I2SIN_OUT4_PORT, I2SIN_OUT4_PIN, GPIO_PIN_RESET);

	/*Configure I/O GPIO pins */
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	GPIO_InitStruct.Pin = IO_2_PIN;
	HAL_GPIO_Init(IO_0_PORT, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = IO_2_PIN;
	HAL_GPIO_Init(IO_1_PORT, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = IO_2_PIN;
	HAL_GPIO_Init(IO_2_PORT, &GPIO_InitStruct);
}

static void setup_hw( void )
{
	/* Configure Flash prefetch and Instruction cache through ART accelerator. */
#if( ART_ACCLERATOR_ENABLE != 0 )
{
	__HAL_FLASH_ART_ENABLE();
}
#endif /* ART_ACCLERATOR_ENABLE */
	MPU_Config();
	CPU_CACHE_Enable();
	HAL_Init();
	/* Set Interrupt Group Priority */
	//	HAL_NVIC_SetPriorityGrouping( NVIC_PRIORITYGROUP_4 );

#ifndef DEBUG
	//Disable PLL and switch to HSI (8 MHz), correct PLL setting will take place inside SystemClock_Config()
	if((RCC->CFGR & RCC_CFGR_SWS) == RCC_CFGR_SWS_PLL) {
		RCC->CFGR &= (uint32_t) (~RCC_CFGR_SW);
		while ((RCC->CFGR & RCC_CFGR_SWS) != RCC_CFGR_SWS_HSI);
	}
#endif
	DWT_Delay_Init();
	// Delays to allow PSU to settle
	DWT_Delay_us(100000);
	SystemClock_Config();
	DWT_Delay_us(100000);

	/* Setup EEPROM emulation */
#if (CFG_EE)
	setup_EE();
#endif
	/* Initialize GPIO */
	GPIO_Init();
	// Init configuration
	CFG_Init();
	/* Initialize all configured peripherals */
	DMA_Init();
	I2C2_Init();
	SAI2_Init();
#ifdef X
    if (_cfg.eeprom_emulation) {
    	printf("write EE\r\n");
    	EE_WriteVariable(VirtAddVarTab[0], 0x4a2f);
    	// Read configuration from emulated EEPROM
    	EE_ReadVariable(VirtAddVarTab[0], &VarDataTab[0]);
    	printf("read EE: %04x\r\n", VarDataTab[0]);
    }
#endif
}

static void setup_EE()
{
	/* Unlock the Flash Program Erase controller */
	HAL_FLASH_Unlock();

	/* EEPROM Init */
	if (EE_Init() != EE_OK) {
		printf("EE error\r\n");
		Error_Handler();
	}
}

#ifdef X
#define BOOT_ADDR	0x08000000	// my MCU boot code base address

struct boot_vectable_ {
    uint32_t Initial_SP;
    void (*Reset_Handler)(void);
};

#define BOOTVTAB	((struct boot_vectable_ *)BOOT_ADDR)

#define DVC_APPJMP_CURRENT_ADDRESS 0x8000000

void dvc_appjmp_init_vectors()  __attribute__((optimize("-O0")));
void dvc_appjmp_goto(uint32_t)	__attribute__((optimize("-O0")));
void dvc_appjmp_enable() 	__attribute__((optimize("-O0")));

extern uint32_t g_pfnVectors[];

static uint32_t vectorTable_RAM[256] __attribute__(( aligned(0x200ul) ));
static uint8_t enabled=0;


void dvc_appjmp_init_vectors()
{
	__disable_irq();
	memmove(vectorTable_RAM,g_pfnVectors,256*sizeof(uint32_t));
	SCB->VTOR = 0x8020000;
	__DSB();
	__ISB();
	__enable_irq();
}

void dvc_appjmp_goto(uint32_t app_addr)
{
//	if(enabled != 1)
//	   return;
printf("JumpToBootLoader\r\n");
	__disable_irq();
	for (int i = 0; i < sizeof(NVIC->ICER) / sizeof(NVIC->ICER[0]); i++) {
		NVIC->ICER[i]=0xFFFFFFFF;
		NVIC->ICPR[i]=0xFFFFFFFF;
	}
	__set_CONTROL(0);
	__set_MSP(*(__IO uint32_t*)app_addr);
	uint32_t JumpAddress = *((volatile uint32_t*) (app_addr + 4));
	__ISB();
	__DSB();
	SysTick->CTRL &= ~(SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk);
	SCB->VTOR = app_addr + 4;
	/* Re-enable all interrupts */
	__enable_irq();
	void (*reset_handler)(void) = (void*)JumpAddress;
	while(1)
	   reset_handler();
}

void dvc_appjmp_enable()
{
	enabled = 1;
}

void switchToDFUBootloader(void)
{
  #define dfuBootloadAddress 0x08000000

  void (*SysMemBootJump)(void);

  SysMemBootJump = (void (*)(void)) (*((uint32_t *) (dfuBootloadAddress + 4)));
  __set_MSP(*(uint32_t *)dfuBootloadAddress);

  SYSCFG->MEMRMP = 0x01;

  SysMemBootJump();

  while (1);
}

void JumpToBootloader(void)
{
printf("JumpToBootLoader\r\n");

	/* Disable all interrupts */
	__disable_irq();

	/* Set the clock to the default state */
	HAL_RCC_DeInit();

	/* Disable Systick timer */
	SysTick->CTRL = 0;
	SysTick->LOAD = 0;
	SysTick->VAL = 0;

	/* Clear Interrupt Enable Register & Interrupt Pending Register */
	for (int i = 0; i < sizeof(NVIC->ICER) / sizeof(NVIC->ICER[0]); i++) {
		NVIC->ICER[i]=0xFFFFFFFF;
		NVIC->ICPR[i]=0xFFFFFFFF;
	}

	// REMAP
	SYSCFG->MEMRMP = 0x01;

	/* Re-enable all interrupts */
	__enable_irq();
#ifdef X
	// Set the MSP
	__set_MSP(BOOTVTAB->Initial_SP);

	// Jump to app firmware
	BOOTVTAB->Reset_Handler();
#else
	volatile uint32_t BootAddr = BOOT_ADDR;
	void (*SysMemBootJump)(void);
	/* Set up the jump to bootloader address + 4 */
	SysMemBootJump = (void (*)(void)) (*((uint32_t *) ((BootAddr + 4))));

	/* Set the main stack pointer to the bootloader stack */
	__set_MSP(*(uint32_t *)BootAddr);

	/* Call the function to jump to bootloader location */
	SysMemBootJump();
#endif
}
/* Set the enumeration of the STM32 Microcontrollers Series*/
enum{C0, F030x8, F030xC, F03xx, F05, F07, F09, F10xx, F105, F107, F10XL, F2, F3, F4, F7, G0, G4, H503, H563, H573, H7x, H7A, H7B, L0, L1, L4, L5, WBA, WBX, WL, U5};

#define MCU			F7		//Define here the MCU being used

void JumpToBootloader (void)
{
	printf("JumpToBooLoader\r\n");
uint32_t i=0;
void (*SysMemBootJump)(void);


/* Set a vector addressed with STM32 Microcontrollers names */
/* Each vector position contains an address to the boot loader entry point */

	volatile uint32_t BootAddr[33];

	BootAddr[C0]     = 0x1FFF0000;
	BootAddr[F030x8] = 0x1FFFEC00;
	BootAddr[F030xC] = 0x1FFFD800;
	BootAddr[F03xx]  = 0x1FFFEC00;
	BootAddr[F05]    = 0x1FFFEC00;
	BootAddr[F07]    = 0x1FFFC800;
	BootAddr[F09]    = 0x1FFFD800;
	BootAddr[F10xx]  = 0x1FFFF000;
	BootAddr[F105]   = 0x1FFFB000;
	BootAddr[F107]   = 0x1FFFB000;
	BootAddr[F10XL]  = 0x1FFFE000;
	BootAddr[F2]     = 0x1FFF0000;
	BootAddr[F3]     = 0x1FFFD800;
	BootAddr[F4]     = 0x1FFF0000;
//	BootAddr[F7]     = 0x1FF00000;
	BootAddr[F7]     = 0x08000000;
	BootAddr[G0]     = 0x1FFF0000;
	BootAddr[G4]     = 0x1FFF0000;
	BootAddr[H503]   = 0x0BF87000;
	BootAddr[H563]   = 0x0BF97000;
	BootAddr[H573]   = 0x0BF97000;
	BootAddr[H7x]    = 0x1FF09800;
	BootAddr[H7A]    = 0x1FF0A800;
	BootAddr[H7B]    = 0x1FF0A000;
	BootAddr[L0]     = 0x1FF00000;
	BootAddr[L1]     = 0x1FF00000;
	BootAddr[L4]     = 0x1FFF0000;
	BootAddr[L5]     = 0x0BF90000;
	BootAddr[WBA]    = 0x0BF88000;
	BootAddr[WBX]    = 0x1FFF0000;
	BootAddr[WL]     = 0x1FFF0000;
	BootAddr[U5]     = 0x0BF90000;

	/* Disable all interrupts */
	__disable_irq();

	/* Disable Systick timer */
	SysTick->CTRL = 0;

	/* Set the clock to the default state */
	HAL_RCC_DeInit();

	/* Clear Interrupt Enable Register & Interrupt Pending Register */
	for (i = 0; i < sizeof(NVIC->ICER) / sizeof(NVIC->ICER[0]); i++) {
		NVIC->ICER[i]=0xFFFFFFFF;
		NVIC->ICPR[i]=0xFFFFFFFF;
	}

	// REMAP
	SYSCFG->MEMRMP = 0x01;

	/* Re-enable all interrupts */
	__enable_irq();

	/* Set up the jump to boot loader address + 4 */
	SysMemBootJump = (void (*)(void)) (*((uint32_t *) ((BootAddr[MCU] + 4))));

	/* Set the main stack pointer to the boot loader stack */
	__set_MSP(*(uint32_t *)BootAddr[MCU]);

	/* Call the function to jump to boot loader location */
	SysMemBootJump();

	/* Jump is done successfully */
	while (1)
	{
		/* Code should never reach this loop */
	}
}
#endif

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
/* Called if a call to pvPortMalloc() fails because there is insufficient
free memory available in the FreeRTOS heap.  pvPortMalloc() is called
internally by FreeRTOS API functions that create tasks, queues, software
timers, and semaphores.  The size of the FreeRTOS heap is set by the
configTOTAL_HEAP_SIZE configuration constant in FreeRTOSConfig.h. */

	/* Force an assert. */
	configASSERT( ( volatile void * ) NULL );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( TaskHandle_t pxTask, char *pcTaskName )
{
	( void ) pcTaskName;
	( void ) pxTask;

/* Run time stack overflow checking is performed if
configCHECK_FOR_STACK_OVERFLOW is defined to 1 or 2.  This hook
function is called if a stack overflow is detected. */

	/* Force an assert. */
	configASSERT( ( volatile void * ) NULL );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	volatile size_t xFreeHeapSpace;

/* This is just a trivial example of an idle hook.  It is called on each
cycle of the idle task.  It must *NOT* attempt to block.  In this case the
idle task just queries the amount of FreeRTOS heap that remains.  See the
memory management section on the http://www.FreeRTOS.org web site for memory
management options.  If there is a lot of heap memory free then the
configTOTAL_HEAP_SIZE value in FreeRTOSConfig.h can be reduced to free up
RAM. */
	xFreeHeapSpace = xPortGetFreeHeapSize();

	/* Remove compiler warning about xFreeHeapSpace being set but never used. */
	( void ) xFreeHeapSpace;
}
/*-----------------------------------------------------------*/

void vAssertCalled( uint32_t ulLine, const char *pcFile )
{
	volatile unsigned long ul = 0;

	( void ) pcFile;
	( void ) ulLine;

	taskENTER_CRITICAL();
	{
		/* Set ul to a non-zero value using the debugger to step out of this
		function. */
		while( ul == 0 )
		{
			__NOP();
		}
	}
	taskEXIT_CRITICAL();
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
	/* User can add his own implementation to report the HAL error return state */
	__disable_irq();
	while (1)
	{
	}
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
