/**
  ******************************************************************************
  * @file    usbd_composite.c
  * @brief   This file provides the USB composite device core functions.
  *
  * @verbatim
  *
  *          ===================================================================
  *                                COMPOSITE Class  Description
  *          ===================================================================
  *
  * @note     In HS mode and when the DMA is used, all variables and data structures
  *           dealing with the DMA during the transaction process should be 32-bit aligned.
  *
  *  @endverbatim
  *
  ******************************************************************************
  */

#include "usbd_composite.h"
#include "usbd_ctlreq.h"
#include "usbd_customhid.h"

static uint8_t USBD_COMPOSITE_Init(USBD_HandleTypeDef *pdev, uint8_t cfgidx);
static uint8_t USBD_COMPOSITE_DeInit(USBD_HandleTypeDef *pdev, uint8_t cfgidx);
static uint8_t USBD_COMPOSITE_Setup(USBD_HandleTypeDef *pdev, USBD_SetupReqTypedef *req);
static uint8_t USBD_COMPOSITE_DataIn(USBD_HandleTypeDef *pdev, uint8_t epnum);
static uint8_t USBD_COMPOSITE_DataOut(USBD_HandleTypeDef *pdev, uint8_t epnum);
static uint8_t USBD_COMPOSITE_EP0_RxReady(USBD_HandleTypeDef *pdev);
static uint8_t USBD_COMPOSITE_EP0_TxReady(USBD_HandleTypeDef *pdev);
static uint8_t USBD_COMPOSITE_SOF(USBD_HandleTypeDef *pdev);
static uint8_t USBD_COMPOSITE_IsoINIncomplete(USBD_HandleTypeDef *pdev, uint8_t epnum);
static uint8_t USBD_COMPOSITE_IsoOutIncomplete(USBD_HandleTypeDef *pdev, uint8_t epnum);

static uint8_t *USBD_COMPOSITE_GetHSCfgDesc(uint16_t *length);
static uint8_t *USBD_COMPOSITE_GetFSCfgDesc(uint16_t *length);
static uint8_t *USBD_COMPOSITE_GetOtherSpeedCfgDesc(uint16_t *length);
static uint8_t *USBD_COMPOSITE_GetDeviceQualifierDesc(uint16_t *length);

USBD_ClassTypeDef USBD_COMPOSITE =
{
	USBD_COMPOSITE_Init,
	USBD_COMPOSITE_DeInit,
	USBD_COMPOSITE_Setup,
	USBD_COMPOSITE_EP0_TxReady,
	USBD_COMPOSITE_EP0_RxReady,
	USBD_COMPOSITE_DataIn,
	USBD_COMPOSITE_DataOut,
	USBD_COMPOSITE_SOF,
	USBD_COMPOSITE_IsoINIncomplete,
	USBD_COMPOSITE_IsoOutIncomplete,
	USBD_COMPOSITE_GetHSCfgDesc,
	USBD_COMPOSITE_GetFSCfgDesc,
	USBD_COMPOSITE_GetOtherSpeedCfgDesc,
	USBD_COMPOSITE_GetDeviceQualifierDesc,
};

typedef struct USBD_COMPOSITE_CFG_DESC_t
{
	uint8_t CONFIG_DESC[USB_CONF_DESC_SIZE];
#ifdef USBD_USE_AUDIO
	uint8_t USBD_AUDIO_DESC[USBD_AUDIO_CONFIG_DESC_SIZE - 0x09];
#endif
#ifdef USBD_USE_CUSTOM_HID
	uint8_t USBD_HID_CUSTOM_DESC[USB_CUSTOM_HID_CONFIG_DESC_SIZ - 0x09];
#endif
} __PACKED USBD_COMPOSITE_CFG_DESC_t;

#if defined(__ICCARM__) /*!< IAR Compiler */
#pragma data_alignment = 4
#endif
__ALIGN_BEGIN USBD_COMPOSITE_CFG_DESC_t USBD_COMPOSITE_HSCfgDesc __ALIGN_END;
//uint8_t USBD_Track_String_Index = (USBD_IDX_INTERFACE_STR + 1);
uint8_t USBD_Track_String_Index = 0;

#if defined(__ICCARM__) /*!< IAR Compiler */
#pragma data_alignment = 4
#endif
/* USB Standard Device Descriptor */
__ALIGN_BEGIN static uint8_t USBD_COMPOSITE_DeviceQualifierDesc[USB_LEN_DEV_QUALIFIER_DESC] __ALIGN_END =
{
	USB_LEN_DEV_QUALIFIER_DESC,
	USB_DESC_TYPE_DEVICE_QUALIFIER,
	0x00,
	0x02,
	0xEF,
	0x02,
	0x01,
	0x40,
	0x01,
	0x00,
};

/**
  * @brief  USBD_COMPOSITE_Init
  *         Initialize the COMPOSITE interface
  * @param  pdev: device instance
  * @param  cfgidx: Configuration index
  * @retval status
  */
static uint8_t USBD_COMPOSITE_Init(USBD_HandleTypeDef *pdev, uint8_t cfgidx)
{
	if (_cfg.debug) {
		printf("COMPOSITE_Init\r\n");
	}
#ifdef USBD_USE_AUDIO
	USBD_AUDIO.Init(pdev, cfgidx);
#endif
#ifdef USBD_USE_CUSTOM_HID
	USBD_CUSTOM_HID.Init(pdev, cfgidx);
#endif

	return (uint8_t)USBD_OK;
}

/**
  * @brief  USBD_COMPOSITE_Init
  *         DeInitialize the COMPOSITE layer
  * @param  pdev: device instance
  * @param  cfgidx: Configuration index
  * @retval status
  */
static uint8_t USBD_COMPOSITE_DeInit(USBD_HandleTypeDef *pdev, uint8_t cfgidx)
{
	if (_cfg.debug) {
		printf("COMPOSITE_DeInit\r\n");
	}
#ifdef USBD_USE_AUDIO
	USBD_AUDIO_DeInit(pdev, cfgidx);
#endif
#ifdef USBD_USE_CUSTOM_HID
	USBD_CUSTOM_HID.DeInit(pdev, cfgidx);
#endif

	return (uint8_t)USBD_OK;
}

/**
  * @brief  USBD_COMPOSITE_Setup
  *         Handle the COMPOSITE specific requests
  * @param  pdev: instance
  * @param  req: usb requests
  * @retval status
  */
static uint8_t USBD_COMPOSITE_Setup(USBD_HandleTypeDef *pdev,
                                    USBD_SetupReqTypedef *req)
{
	if (_cfg.debug) {
		printf("COMPOSITE_Setup %02x %02x %04x %04x %d %d\r\n", req->bmRequest, req->bRequest, req->wIndex, req->wValue, req->wLength, pdev->dev_state);
	}
#ifdef USBD_USE_AUDIO
	if (LOBYTE(req->wIndex) == AUDIO_AC_ITF_NBR || LOBYTE(req->wIndex) == AUDIO_IN_AS_ITF_NBR) {
		return USBD_AUDIO.Setup(pdev, req);
	}
#endif
#ifdef USBD_USE_CUSTOM_HID
	if (LOBYTE(req->wIndex) == CUSTOM_HID_ITF_NBR || LOBYTE(req->wIndex) == CUSTOM_HID_IN_EP) {
		return USBD_CUSTOM_HID.Setup(pdev, req);
	}
#endif

	return USBD_FAIL;
}

/**
  * @brief  USBD_COMPOSITE_DataIn
  *         handle data IN Stage
  * @param  pdev: device instance
  * @param  epnum: endpoint index
  * @retval status
  */
static uint8_t USBD_COMPOSITE_DataIn(USBD_HandleTypeDef *pdev, uint8_t epnum)
{
#ifdef USBD_USE_AUDIO
	if (epnum == (AUDIO_IN_EP & EP_ADDR_MSK)) {
		return USBD_AUDIO.DataIn(pdev, epnum);
	}
#endif
#ifdef USBD_USE_CUSTOM_HID
	if (epnum == (CUSTOM_HID_IN_EP & EP_ADDR_MSK)) {
		return USBD_CUSTOM_HID.DataIn(pdev, epnum);
	}
#endif

	return USBD_FAIL;
}

/**
  * @brief  USBD_COMPOSITE_EP0_RxReady
  *         handle EP0 Rx Ready event
  * @param  pdev: device instance
  * @retval status
  */
static uint8_t USBD_COMPOSITE_EP0_RxReady(USBD_HandleTypeDef *pdev)
{
	if (_cfg.debug) {
		printf("COMPOSITE_EP0_RxReady\r\n");
	}
#ifdef USBD_USE_AUDIO
	if (LOBYTE(pdev->request.wIndex) == AUDIO_AC_ITF_NBR || LOBYTE(pdev->request.wIndex) == AUDIO_IN_AS_ITF_NBR) {
		USBD_AUDIO.EP0_RxReady(pdev);
	}
#endif
#ifdef USBD_USE_CUSTOM_HID
	if (LOBYTE(pdev->request.wIndex) == CUSTOM_HID_ITF_NBR || LOBYTE(pdev->request.wIndex) == CUSTOM_HID_IN_EP) {
		USBD_CUSTOM_HID.EP0_RxReady(pdev);
	}
#endif

	return (uint8_t)USBD_OK;
}

/**
  * @brief  USBD_COMPOSITE_EP0_TxReady
  *         handle EP0 TRx Ready event
  * @param  pdev: device instance
  * @retval status
  */
static uint8_t USBD_COMPOSITE_EP0_TxReady(USBD_HandleTypeDef *pdev)
{
#ifdef USBD_USE_AUDIO
	USBD_AUDIO.EP0_TxSent(pdev);
#endif

	return (uint8_t)USBD_OK;
}

/**
  * @brief  USBD_COMPOSITE_SOF
  *         handle SOF event
  * @param  pdev: device instance
  * @retval status
  */
static uint8_t USBD_COMPOSITE_SOF(USBD_HandleTypeDef *pdev)
{
#ifdef USBD_USE_AUDIO
	USBD_AUDIO.SOF(pdev);
#endif

	return (uint8_t)USBD_OK;
}

/**
  * @brief  USBD_COMPOSITE_IsoINIncomplete
  *         handle data ISO IN Incomplete event
  * @param  pdev: device instance
  * @param  epnum: endpoint index
  * @retval status
  */
static uint8_t USBD_COMPOSITE_IsoINIncomplete(USBD_HandleTypeDef *pdev, uint8_t epnum)
{
#ifdef USBD_USE_AUDIO
	if (epnum == (AUDIO_IN_EP & EP_ADDR_MSK)) {
		USBD_AUDIO.IsoINIncomplete(pdev, epnum);
	}
#endif

	return (uint8_t)USBD_OK;
}

/**
  * @brief  USBD_COMPOSITE_IsoOutIncomplete
  *         handle data ISO OUT Incomplete event
  * @param  pdev: device instance
  * @param  epnum: endpoint index
  * @retval status
  */
static uint8_t USBD_COMPOSITE_IsoOutIncomplete(USBD_HandleTypeDef *pdev, uint8_t epnum)
{
#ifdef USBD_USE_AUDIO
	if (epnum == AUDIO_OUT_EP) {
		USBD_AUDIO.IsoOUTIncomplete(pdev, epnum);
	}
#endif

	return (uint8_t)USBD_OK;
}
/**
  * @brief  USBD_COMPOSITE_DataOut
  *         handle data OUT Stage
  * @param  pdev: device instance
  * @param  epnum: endpoint index
  * @retval status
  */
static uint8_t USBD_COMPOSITE_DataOut(USBD_HandleTypeDef *pdev, uint8_t epnum)
{
#ifdef USBD_USE_AUDIO
	if (epnum == AUDIO_OUT_EP) {
		return USBD_AUDIO.DataOut(pdev, epnum);
	}
#endif
#ifdef USBD_USE_CUSTOM_HID
	if (epnum == CUSTOM_HID_OUT_EP) {
		return USBD_CUSTOM_HID.DataOut(pdev, epnum);
	}
#endif

	return USBD_FAIL;
}

/**
  * @brief  USBD_COMPOSITE_GetHSCfgDesc
  *         return configuration descriptor
  * @param  length : pointer data length
  * @retval pointer to descriptor buffer
  */
static uint8_t *USBD_COMPOSITE_GetHSCfgDesc(uint16_t *length)
{
	*length = (uint16_t)sizeof(USBD_COMPOSITE_HSCfgDesc);
	if (_cfg.debug) {
		printf("COMPOSITE_GetHSCfgDesc: %d\r\n", *length);
	}
	return (uint8_t *)&USBD_COMPOSITE_HSCfgDesc;
}

/**
  * @brief  USBD_COMPOSITE_GetFSCfgDesc
  *         return configuration descriptor
  * @param  length : pointer data length
  * @retval pointer to descriptor buffer
  */
static uint8_t *USBD_COMPOSITE_GetFSCfgDesc(uint16_t *length)
{
	*length = (uint16_t)sizeof(USBD_COMPOSITE_HSCfgDesc);
	return (uint8_t *)&USBD_COMPOSITE_HSCfgDesc;
}

/**
  * @brief  USBD_COMPOSITE_GetOtherSpeedCfgDesc
  *         return configuration descriptor
  * @param  length : pointer data length
  * @retval pointer to descriptor buffer
  */
static uint8_t *USBD_COMPOSITE_GetOtherSpeedCfgDesc(uint16_t *length)
{
	*length = (uint16_t)sizeof(USBD_COMPOSITE_HSCfgDesc);
	return (uint8_t *)&USBD_COMPOSITE_HSCfgDesc;
}

/**
  * @brief  DeviceQualifierDescriptor
  *         return Device Qualifier descriptor
  * @param  length : pointer data length
  * @retval pointer to descriptor buffer
  */
uint8_t *USBD_COMPOSITE_GetDeviceQualifierDesc(uint16_t *length)
{
	*length = (uint16_t)sizeof(USBD_COMPOSITE_DeviceQualifierDesc);
	return USBD_COMPOSITE_DeviceQualifierDesc;
}

void USBD_COMPOSITE_Mount_Class(void)
{
	uint16_t len = 0;
	uint8_t *ptr = NULL;

	uint8_t in_ep_track = 0x81;
	uint8_t out_ep_track = 0x01;
	uint8_t interface_no_track = 0x00;

#ifdef USBD_USE_AUDIO
	ptr = USBD_AUDIO.GetHSConfigDescriptor(&len);
	USBD_Update_Audio_DESC(ptr, interface_no_track, interface_no_track + 1, out_ep_track, in_ep_track, USBD_Track_String_Index);
	memcpy(USBD_COMPOSITE_HSCfgDesc.USBD_AUDIO_DESC, ptr + 0x09, len - 0x09);
	interface_no_track += 1;
#if (CFG_AUDIO_OUT == 1)
	out_ep_track += 1;
	interface_no_track += 1;
#endif
#if (CFG_AUDIO_IN == 1)
	in_ep_track += 1;
	interface_no_track += 1;
#endif
#endif

#ifdef USBD_USE_CUSTOM_HID
	ptr = USBD_CUSTOM_HID.GetHSConfigDescriptor(&len);
	USBD_Update_Custom_HID_DESC(ptr, interface_no_track, in_ep_track, out_ep_track, USBD_Track_String_Index);
	memcpy(USBD_COMPOSITE_HSCfgDesc.USBD_HID_CUSTOM_DESC, ptr + 0x09, len - 0x09);
	interface_no_track += 1;
#endif

	uint16_t CFG_SIZE = sizeof(USBD_COMPOSITE_CFG_DESC_t);
	ptr = USBD_COMPOSITE_HSCfgDesc.CONFIG_DESC;
	/* Configuration Descriptor */
	ptr[0] = 0x09;                        /* bLength: Configuration Descriptor size */
	ptr[1] = USB_DESC_TYPE_CONFIGURATION; /* bDescriptorType: Configuration */
	ptr[2] = LOBYTE(CFG_SIZE);            /* wTotalLength:no of returned bytes */
	ptr[3] = HIBYTE(CFG_SIZE);
	ptr[4] = interface_no_track; /* bNumInterfaces: 2 interface */
	ptr[5] = 0x01;               /* bConfigurationValue: Configuration value */
	ptr[6] = 0x00;               /* iConfiguration: Index of string descriptor describing the configuration */
#if (USBD_SELF_POWERED == 1U)
	ptr[7] = 0xC0; /* bmAttributes: Bus Powered according to user configuration */
#else
	ptr[7] = 0x80; /* bmAttributes: Bus Powered according to user configuration */
#endif
	ptr[8] = USBD_MAX_POWER; /* MaxPower 100 mA */
}
