/**
  ******************************************************************************
  * @file           : usbd_custom_hid_if.c
  * @version        : v1.0_Cube
  * @brief          : USB Device Custom HID interface file.
  ******************************************************************************
  * @attention
  */

#include "usbd_custom_hid_if.h"
#include "queue_cmd.h"
#include "appjmp.h"

#define HID_GET_STATUS		0x01
#define HID_SET_REGISTER	0x02
#define HID_SET_FEATURE		0x03

#define HID_SET_ADC_CLIP_DETECTION	0x01
#define HID_SET_ADC_PEAK_LEVEL_CH1	0x02
#define HID_SET_ADC_PEAK_LEVEL_CH2	0x03
#define HID_SET_ADC_MONO			0x04
#define HID_SET_ADC_FILTER			0x05
#define HID_SAVE_CONFIG				0x06
#define HID_RESET					0x07
#define HID_DFU						0x08

#define SWITCH_TO_DFU	0x01234567

extern USBD_HandleTypeDef hUsbDevice;

/** Usb custom HID report descriptor. */
__ALIGN_BEGIN static uint8_t CUSTOM_HID_ReportDesc_HS[USBD_CUSTOM_HID_REPORT_DESC_SIZE] __ALIGN_END =
{
		0x06, 0x00, 0xff, // Usage Page(Undefined )
		0x09, 0x01, // USAGE (Undefined)
		0xa1, 0x01, // COLLECTION (Application)
		0x15, 0x00, // LOGICAL_MINIMUM (0)
		0x26, 0xff, 0x00, // LOGICAL_MAXIMUM (255)
		0x75, 0x08, // REPORT_SIZE (8)
		0x95, 0x06, // REPORT_COUNT (6)
		0x85, 0x01, // REPORT_ID 0x01
		0x09, 0x01, // USAGE (Undefined)
		0x81, 0x02, // INPUT (Data,Var,Abs)
		0x95, 0x04, // REPORT_COUNT (4)
		0x85, 0x02, // REPORT_ID 0x02
		0x09, 0x01, // USAGE (Undefined)
		0x91, 0x02, // OUTPUT (Data,Var,Abs)
		0x95, 0x03, // REPORT_COUNT (3)
		0x85, 0x03, // REPORT_ID 0x03
		0x09, 0x01, // USAGE (Undefined)
		0xb1, 0x02, // FEATURE (Data,Var,Abs)
		0xC0 /* END_COLLECTION */
};

static uint8_t tx_buffer[64];		//Variable to store the output data

static int8_t CUSTOM_HID_Init_HS(void);
static int8_t CUSTOM_HID_DeInit_HS(void);
static int8_t CUSTOM_HID_OutEvent_HS(uint8_t* state);
#ifdef USBD_CUSTOMHID_CTRL_REQ_COMPLETE_CALLBACK_ENABLED
static int8_t CUSTOM_HID_CtrlReqComplete_HS(uint8_t request, uint16_t wLength);
#endif /* USBD_CUSTOMHID_CTRL_REQ_COMPLETE_CALLBACK_ENABLED */

#ifdef USBD_CUSTOMHID_CTRL_REQ_GET_REPORT_ENABLED
static uint8_t *CUSTOM_HID_GetReport_HS(uint8_t reportId, uint16_t *ReportLength);
#endif /* USBD_CUSTOMHID_CTRL_REQ_GET_REPORT_ENABLED */

USBD_CUSTOM_HID_ItfTypeDef USBD_CustomHID_fops_HS =
{
  CUSTOM_HID_ReportDesc_HS,
  CUSTOM_HID_Init_HS,
  CUSTOM_HID_DeInit_HS,
  CUSTOM_HID_OutEvent_HS,
#ifdef USBD_CUSTOMHID_CTRL_REQ_COMPLETE_CALLBACK_ENABLED
  CUSTOM_HID_CtrlReqComplete_HS,
#endif /* USBD_CUSTOMHID_CTRL_REQ_COMPLETE_CALLBACK_ENABLED */
#ifdef USBD_CUSTOMHID_CTRL_REQ_GET_REPORT_ENABLED
  CUSTOM_HID_GetReport_HS,
#endif /* USBD_CUSTOMHID_CTRL_REQ_GET_REPORT_ENABLED */
};

static int8_t hidSetRegister(uint8_t *params);
static int8_t hidSetFeature(uint8_t *params);

extern uint32_t dfu_mode;

/**
  * @brief  Initializes the CUSTOM HID media low layer
  * @retval USBD_OK if all operations are OK else USBD_FAIL
  */
static int8_t CUSTOM_HID_Init_HS(void)
{
	for (uint8_t i = 0; i < 64; i++) {
		tx_buffer[i] = i;
	}
  return (USBD_OK);
}

/**
  * @brief  DeInitializes the CUSTOM HID media low layer
  * @retval USBD_OK if all operations are OK else USBD_FAIL
  */
static int8_t CUSTOM_HID_DeInit_HS(void)
{
  return (USBD_OK);
}

/**
  * @brief  Manage the CUSTOM HID class events
  * @param  event_idx: Event index
  * @param  state: Event state
  * @retval USBD_OK if all operations are OK else USBD_FAIL
  */
static int8_t CUSTOM_HID_OutEvent_HS(uint8_t* state)
{
	if (state[0] == HID_SET_REGISTER) {
		if (_cfg.debug) {
			printf("report %d %02x %02x\r\n", HID_SET_REGISTER, state[1], state[2]);
		}
		hidSetRegister(&state[1]);
	}
	else if (state[0] == HID_SET_FEATURE) {
		if (_cfg.debug) {
			printf("report %d %02x %02x %02x\r\n", HID_SET_FEATURE, state[1], state[2], state[3]);
		}
		hidSetFeature(&state[1]);
	}
	return (USBD_OK);
}

static int8_t hidSetRegister(uint8_t *params)
{
	sendQueueCmd(audio_in_queue, SET_REGISTER, &params[1]);
	return (USBD_OK);
}

static int8_t hidSetFeature(uint8_t *params)
{
	switch (params[0]) {
	case HID_SET_ADC_CLIP_DETECTION:
		sendQueueCmd(audio_in_queue, CLIP_DETECTION, &params[1]);
		break;
	case HID_SET_ADC_PEAK_LEVEL_CH1:
		sendQueueCmd(audio_in_queue, PEAK_LEVEL_CH1, &params[1]);
		break;
	case HID_SET_ADC_PEAK_LEVEL_CH2:
		sendQueueCmd(audio_in_queue, PEAK_LEVEL_CH2, &params[1]);
		break;
	case HID_SET_ADC_MONO:
		sendQueueCmd(audio_in_queue, MONO, &params[1]);
		break;
	case HID_SET_ADC_FILTER:
		sendQueueCmd(audio_in_queue, FILTER, &params[1]);
		break;
	case HID_SAVE_CONFIG:
		sendQueueCmd(audio_in_queue, SAVE_CONFIG, &params[1]);
		break;
	case HID_RESET:
		NVIC_SystemReset();
		break;
	case HID_DFU:
		dfu_mode = JUMP_TO_DFU;
		NVIC_SystemReset();
		break;
	default:
		break;
	}
	return (USBD_OK);
}

#ifdef USBD_CUSTOMHID_CTRL_REQ_COMPLETE_CALLBACK_ENABLED
/**
  * @brief  CUSTOM_HID_CtrlReqComplete_HS
  *         Manage the CUSTOM HID control request complete
  * @param  request: control request
  * @param  wLength: request wLength
  * @retval Result of the operation: USBD_OK if all operations are OK else USBD_FAIL
  */
static int8_t CUSTOM_HID_CtrlReqComplete_HS(uint8_t request, uint16_t wLength)
{
  UNUSED(wLength);
  switch (request)
  {
    case CUSTOM_HID_REQ_SET_REPORT:

      break;

    case CUSTOM_HID_REQ_GET_REPORT:

      break;

    default:
      break;
  }

  return (0);
}
#endif /* USBD_CUSTOMHID_CTRL_REQ_COMPLETE_CALLBACK_ENABLED */

#ifdef USBD_CUSTOMHID_CTRL_REQ_GET_REPORT_ENABLED
/**
  * @brief  CUSTOM_HID_GetReport_HS
  *         Manage the CUSTOM HID control Get Report request
  * @param  event_idx: event index
  * @param  state: event state
  * @retval return pointer to HID report
  */
static uint8_t *CUSTOM_HID_GetReport_HS(uint8_t reportId, uint16_t *reportLength)
{
	uint8_t *pbuf = tx_buffer;
	if (reportId == HID_GET_STATUS) {
		*reportLength = 7;

	}
	return (pbuf);
}
#endif /* USBD_CUSTOMHID_CTRL_REQ_GET_REPORT_ENABLED */

/**
  * @brief  Send the report to the Host
  * @param  report: The report to be sent
  * @param  len: The report length
  * @retval USBD_OK if all operations are OK else USBD_FAIL
  */
/*
static int8_t USBD_CUSTOM_HID_SendReport_HS(uint8_t *report, uint16_t len)
{
  return USBD_CUSTOM_HID_SendReport(&hUsbDeviceHS, report, len);
}
*/
